/*
 *  Kylin Box security
 *
 *  This file contains apis for Kylin Box security libraries
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#ifndef BOX_LIB_INCLUDE_H
#define BOX_LIB_INCLUDE_H

#include <linux/limits.h>
#include <sys/types.h>
#include <stdbool.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"{
#endif

/* 
 * 加密方案参数
 */
#define NORMAL          0X00000000 
#define ECRYPTFS        0X01000000

#define AES256          0X00010000
#define AES128          0X00020000
#define AES196          0X00030000
#define BLOWFISH128     0X00040000
#define BLOWFISH256     0X00050000
#define BLOWFISH448     0X00060000
#define TWOFISH192      0X00070000
#define SM4             0X00080000

/* 
 * box 类型,用于搜索
 */
#define BOX_NORMAL      0b00000001
#define BOX_ENCRYPTED   0b00000010
#define BOX_SHARED      0b00000100
#define BOX_UNSHARED    0b00001000
#define BOX_SELF        0b00010000
#define BOX_OTHERS      0b00100000
#define BOX_MOUNTED     0b01000000
#define BOX_NOTMOUNTED  0b10000000
#define BOX_TS         0b100000000
#define BOX_ENC_TYPES   (BOX_ENCRYPTED|BOX_TS)
#define BOX_ALL         (BOX_NORMAL|BOX_ENCRYPTED|BOX_SHARED|BOX_UNSHARED|\
                        BOX_SELF|BOX_OTHERS|BOX_MOUNTED|BOX_NOTMOUNTED|BOX_TS)


/* 
 * 密码复杂度配置文件
 */
#define BOX_PWQUALITY_CONFIG "/etc/box/box_pwquality.conf"

/* 
 * errno扩展
 */
#define EBOXDISABLED        255         /* BOX is DISABLED */
#define EINVALUSERNAME      254         /* USERNAME is invalid */
#define EUSERNAMETOOLONG    253         /* USERNAME is too long */
#define EUNINITIALIZED      252         /* BOX is NOT initialized */
#define ECONFBROKEN         251         /* conf is broken */
#define EWRITECONF          250         /* writing conf failed */
#define EMOUNTEDCONF        249         /* mountpoints conf broken */
#define EKEYTOOLONG         248         /* passphrase is too long */
#define EFNEK_SUPPORT       247         /* kernel does not support filename encryption */
#define EBOXISMOUNTED       246         /* box is already mounted */
#define EBOXISENCRYPTED     245         /* box is already encrypted */
#define EDELDIRFAILED       244         /* del files failed */
#define EMOUNTED            243         /* mounted failed */
#define EMOVEDATA           242         /* moving date failed */
#define EUMOUNT             241         /* umount failed */
#define ENOTMOUNT           240         /* not mounted */
#define ENOTENCRYPTED       239         /* box is not encrypted */
#define EISMOUNTED          238         /* box is mounted */
#define EINVALNAME          237         /* invalid user name */
#define EPASSWORD           236         /* password is incorrect */
#define EINVALBOXNAME       235         /* invalid box name*/
#define EINVALPASS          234         /* invalid passphrase */
#define NONSUPPORT          233         /* does not support root users */
#define ECHECKSUM           232         /* checksum error */
#define ERESETKEY           231         /* no support reset key */
#define EBOXNAMETOOLONG     230
#define EZIPERROR           229
#define EEXTRACT            228
#define ECOPY               227
#define EBOXPASSERR            226
#define EADDFILETOZIP       225
#define FILE_EXIS_ERR       17          /* 解决界面创建保存问题*/

#define SIGSIZE 16                      /* 与ecryptfs一致 */

#define ECRYPT_CHECK_ON 1
#define ECRYPT_CHECK_OFF 0

#define BOX_ENCRYPTED_DIR   0x0001
#define BOX_DECRYPTED_DIR   0x0002
#define BOX_NONE_DIR        0x0004      /* normal box, non ecryptfs */

enum BoxEvent{
    CreatBoxEvent = 1,
    DeleteBoxEvent ,
    RenameBoxEvent,
    MountBoxEvent,
    UmountBoxEvent,
};

struct box_info{
    char name[256];
    char path[256];
    char owner_name[64];
    uint16_t box_type;
};

struct box_info_list{
    struct box_info info;
    struct box_info_list *next;
};

typedef enum _Box_Conf_Type {
    passwd_status = 1,
    passwd_conf,
    passwd_minlen,
    passwd_minclass,
    passwd_usercheck,
    faillock_status,
    faillock_deny,
    faillock_unlock_time,
    faillock_max_tries,
    lockidle_status,
    autolock_status,
    autolock_timeout,
}Box_Conf_Type;


/**
 * @brief box_create 创建普通保护箱接口
 * @param box_name [in] 保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_create(const char *box_name);

/**
 * @brief box_delete 删除保护箱接口
 * @param box_name [in] 保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_delete(const char *box_name);

/**
 * @brief box_change_name 修改保护箱名称接口
 * @param old_box_name [in] 原保护箱名称
 * @param new_box_name [in] 新保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_change_name(const char *old_box_name,const char *new_box_name);

/**
 * @brief box_crypto_create 创建加密保护箱接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 保护箱密码
 * @param passwd_len [in] 密码数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_create(const char *box_name, const char *passwd, int passwd_len);

/**
 * @brief box_transparent_create 创建透明加密保护箱接口
 * @param box_name [in] 保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_transparent_create(const char *box_name);

/**
 * @brief box_crypto_change_passpharse 保护箱用户密钥重置密钥接口
 * @param box_name [in] 保护箱名称
 * @param old_passwd [in] 旧密码
 * @param old_passwd_len [in] 旧密码数据长度
 * @param new_passwd [in] 新密码
 * @param new_passwd_len [in] 新密码数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_change_passpharse(const char *box_name, char *old_passwd, int old_passwd_len, char *new_passwd, int new_passwd_len);

/**
 * @brief box_crypto_decrypt 解除加密保护箱接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 密码
 * @param passwd_len [in] 密码数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_decrypt(const char *box_name, char *passwd, int passwd_len);

/**
 * @brief box_crypto_verify_passphrase 认证密钥合法性接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 密码
 * @param passwd_len [in] 密码数据长度
 * @return 0认证失败，1认证成功
 */
bool box_crypto_verify_passphrase(const char *box_name, const char *passwd, int passwd_len);

/**
 * @brief box_crypto_mount 解锁加密保护箱接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 密码
 * @param passwd_len [in] 密码数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_mount(const char *box_name, char *passwd, int passwd_len);

/**
 * @brief box_crypto_mount_by_global_key 保护箱全局密钥挂载接口
 * @param box_name [in] 保护箱名称
 * @param key [in] 全局密钥
 * @param key_len [in] 全局密钥数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_mount_by_global_key(const char *box_name, unsigned char *key, int key_len);

/**
 * @brief box_crypto_umount 锁定加密保护箱接口
 * @param box_name [in] 保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_umount(const char *box_name);

/**
 * @brief box_crypto_umount_all 锁定全部保护箱接口
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_umount_all();

/**
 * @brief box_crypto_verify ....不清楚作用是啥
 * @param box_name
 * @return 0 成功，其他参加错误码列表
 */
int box_crypto_verify(const char *box_name);

/**
 * @brief box_enabled 检查内核KBOX状态接口
 * @return 0 未开启，1开启
 */
int box_enabled();

/**
 * @brief box_get_info 获取保护箱信息接口
 * @param box_name [in] 保护箱名称
 * @param info [out] 保护箱信息
 * @return 0 成功，其他参见错误码列表
 */
int box_get_info(const char *box_name, struct box_info *info);

/**
 * @brief box_get_list 获取保护箱列表接口
 * @param box_type [in] 保护箱类型
 * @param list [out] 保护箱列表
 * @return 0 成功，其他参见错误码列表
 */
int box_get_list(int box_type,struct box_info_list **list);

/**
 * @brief box_info_list_destroy 销毁保护箱列表接口
 * @param list [in] 保护箱列表
 * @return 0 成功，其他参见错误码列表
 */
int box_info_list_destroy(struct box_info_list *list);

/**
 * @brief box_global_key_is_exist 检查保护箱全局密钥是否存在
 * @return true 存在， false 不存在
 */
bool box_global_key_is_exist();

/**
 * @brief box_create_global_key 创建全局密钥接口
 * @param key [out] 底层反馈的最终key的数据
 * @param len [in/out] 输入表示key内存长度，输出表示key反馈的有效长度
 * @return 0 成功，其他参见错误码列表
 */
int box_create_global_key(unsigned char *key, int *len);

/**
 * @brief box_remove_global_key 删除全局密钥接口
 * @return 0 成功，其他参见错误码列表
 */
int box_remove_global_key();

/**
 * @brief box_verify_global_key 认证全局密钥合法性接口
 * @param key [in] 上层应用输入的全局密钥
 * @param len [in] 全局密钥数据长度
 * @return true 合法，false 不合法
 */
bool box_verify_global_key(unsigned char *key, int len);

/**
 * @brief box_crypto_change_passpharse_by_global_key 保护箱全局密钥重置密钥接口
 * @param box_name [in] 保护箱名称
 * @param key [in] 全局密钥
 * @param key_len [in] 全局密钥数据长度
 * @param passwd [in] 新密钥数据
 * @param passwd_len [in] 新密钥数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_change_passpharse_by_global_key(const char *box_name, unsigned char *key, unsigned int key_len, const char *passwd, int passwd_len);

/**
 * @brief box_crypto_set_resetkey_by_passpharse 保护箱全局密钥功能设置接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 密钥数据
 * @param passws_len [in] 密钥数据长度
 * @return 0 成功，其他参见错误码列表
 */
int box_crypto_set_resetkey_by_passpharse(const char *box_name, const char *passwd, unsigned int passws_len);

/**
 * @brief box_init 保护箱初始化接口
 * @return 0 成功，其他参见错误码列表
 */
int box_init();

/**
 * @brief box_destroy 保护箱逆初始化接口
 * @return 0 成功，其他参见错误码列表
 */
int box_destroy();

/**
 * @brief box_err 保护箱错误信息解析接口
 * @param err [in] 错误码
 * @return 错误信息
 */
char *box_err(int err);

/**
 * @brief box_resetkey_status 文件管理器密钥重置密码检查接口
 * @return true 成功
 */
bool box_resetkey_status();

/**
 * @brief box_setattr 保护箱设置扩展属性接口
 * @param box_name [in] 保护箱名称
 * @return 0 成功，其他参见错误码列表
 */
int box_setattr(const char *box_name);

/**
 * @brief box_passwd_strong_check 保护箱检查密码复杂度接口
 * @param passwd [in] 导入保护箱加密密码
 * @param passwd_len [in] 导入保护箱加密密码长度
 * @param error [in/out] 若校验不通过，返回错误信息，内存接口内申请，外部释放
 * 
 * @return 0~100 密码复杂度分数，其他参见pwquality_strerror()
 */
int box_passwd_strong_check(const char *passwd, int passwd_len, char *auxerror);

/**
 * @brief box_export 导出文件保护箱 
 * @param box_name [in] 保护箱名称
 * @param path [in] 导出保护箱包含导出完整文件名的完整文件路径
 * @param passwd [in] 导出保护箱加密密码
 * 
 * 
 * @return 0成功，其他失败
 */
int box_export(const char *box_name, const char *path, const char *passwd);

/**
 * @brief box_import 导入文件保护箱
 * @param path [in] 导入保护箱文件
 * @param passwd [in] 导入保护箱压缩加密密码
 * @param new_boxname [in] 导入的新保护箱名称，暂时在函数外判断不能重名然后再传入函数内，后续优化
 * @param passwd [in] 导入原保护箱密码，导入透明保护箱时可为NULL
 * 
 * @return 0成功，其他失败
 */
int box_import(const char *path, const char *passwd, const char *new_boxname, const char *boxpasswd);

/**
 * @brief box_crypto_verify_passphrase_v2 认证密钥合法性接口
 * @param box_name [in] 保护箱名称
 * @param passwd [in] 密码
 * @param passwd_len [in] 密码数据长度
 * @param error [out] 提示信息
 * @return 0认证失败，1认证成功
 * @attention *error内存通过malloc分配，调用者需使用free()接口释放内存
 */
bool box_crypto_verify_passphrase_v2(const char *box_name, const char *passwd, int passwd_len, char **error);

/*检测导入解压密码是否正确*/
/**
 * @brief box_import_checkpwd 检测导入解压密码是否正确
 * @param path [in] 导入保护箱文件完成路径
 * @param passwd  导入界面获得用户输入的当前密码
 * @return 0 成功 密码正确，-1 失败 密码错误
 */
int box_import_checkpwd(const char *path, const char *passwd);

/*检测导入的文件是否是加密压缩包*/
/**
 * @brief box_import_checkpwd 检测导入的文件是否是加密压缩文件
 * @param path [in] 导入保护箱文件完成路径
 * @return 0 文件未加密或非保护箱压缩文件,  1 透明加密保护箱压缩加密文件，2 加密保护箱压缩加密文件 -1 函数异常 
 */
int box_import_check_encrypted(const char *path);

/**
 * @brief 创建内置保护箱
 * @param ts_onx [in] 透明保护箱名称
 * @param enc_box [in] 密钥保护箱名称
 * @param passwd [in] 密钥保护箱密码
 * @return 0 成功 其他 失败
 */
int box_create_builtin_box(const char *ts_box, const char *enc_box, const char *passwd);

/**
 * @brief 判断是否为内置保护箱
 * @param box_name [in] 密钥保护箱名称
 * @return true 是 false 不是
 */
bool box_check_builtin_flag(const char *box_name);

/**
 * @brief 移除保护箱内置标识
 * @param box_name [in] 密钥保护箱名称
 * @return 0 成功 其他 失败
 */
int box_remove_builtin_flag(const char *box_name);

/**
 * @brief 获取当前配置模式
 * @param 
 * @return  1 用户配置， 0 全局配置；其他失败
 */
int box_get_configure_mode();

/**
 * @brief 设置当前配置模式
 * @param mode 1 用户模式， 0 全局模式
 * @return  0成功， 其他失败； 
 */
int box_set_configure_mode(int mode);


/**
 * @brief 获取当前int类型配置项
 * @param  type 配置类型枚举，value 获得的值
 * @return  0 成功；其他失败
 */
int box_get_iconfig_item(Box_Conf_Type type, int *value);

/**
 * @brief 设置当前int类型配置项
 * @param  type 配置类型枚举，value 要设置的值
 * @return  0 成功；其他失败
 */
int box_set_iconfig_item(Box_Conf_Type type, int value);

/**
 * @brief 获取当前char类型配置项
 * @param  type 配置类型枚举，item 获取的值，length item最大长度
 * @return  0 成功；其他失败
 */
int box_get_strconfig_item(Box_Conf_Type type, char *item, int length);

/**
 * @brief 设置当前char类型配置项
 * @param  type 配置类型枚举，item 设置值
 * @return  0 成功；其他失败
 */
int box_set_strconfig_item(Box_Conf_Type type, char *item);

/**
 * @brief 重置所有保护箱防暴力破解失败记录
 * @param  
 * @return  0 成功；其他失败
 */
int box_reset_all_tally();

/**
 * @brief 检查当前保护箱升级兼容状态
 * @param  
 * @return  0 正常; 1 升级兼容； 其他失败
 */
int box_check_upgrade_compatibility_status();

/**
 * @brief 保护箱升级兼容
 * @param
 * @return  0 成功； 其他失败
 */
int box_compatibility_upgrade();

/**
 * @brief 检查保护箱全局密钥可用状态
 * @param box_name [in] 密钥保护箱名称
 * @return 0 不可用 1 可用 其他 失败
 */
int box_crypto_check_resetkey_status(const char *box_name);

/**
 * @brief box_dbglog 日志记录接口
 * @param format [in] 格式化字符串
 * @return 无
 */
void box_dbglog(const char *format, ...);

#define BOXLOG(fmt, ...) do { \
    box_dbglog("%s - " fmt, __func__, ##__VA_ARGS__); \
} while(0)

#ifdef __cplusplus
}
#endif
#endif

